import requests
from digitalbrainsdk.api.api.core import Core
from digitalbrainsdk.config import ConfigManager
import os
from treelib import Tree
from IPython.display import display, Markdown
import json


class RegionApi:
    def __init__(self, environment="PRODUCTION", species="mouse"):
        self.core = Core(environment)
        self.region_data = None
        self.region_type = None
        self.species = species
        cache_dir_name = ConfigManager().get("CACHE", "CacheDir")
        self.cache_dir = None
        if not self.cache_dir:
            self.cache_dir = os.path.join(os.getcwd(), cache_dir_name, "region")
        if not os.path.exists(self.cache_dir):
            os.makedirs(self.cache_dir)

    def get_region(self, id=None, file=None):
        if not self.region_data:
            self._download_region_data()
        target_region = None
        if self.region_data:
            if id:
                target_region = self._get_region_by_id(id)
            elif file:
                target_region = self._get_region_by_file(file)
        if target_region:
            content = self._download_region(target_region)
            file_path = f"{self.cache_dir}/{target_region['file']}"
            with open(file_path, "wb") as f:
                f.write(content)
            return file_path
        return None

    def _download_region_data(self):
        # Check if species and project are valid
        if self.species:
            # Define the cache file path
            cache_file_path = (
                f"{self.cache_dir}/{self.species}_region.json"
            )

            # Check if the cache file exists
            if os.path.exists(cache_file_path):
                print(f"Loading cached data from {cache_file_path}")
                # Load the cached data
                with open(cache_file_path, "r") as cache_file:
                    cached_data = json.load(cache_file)
                    self.region_data = cached_data["region_data"]
                    self.region_type = cached_data["region_type"]
            else:
                # Fetch the data from the URL
                url = f"{self.core.base_url}/info/{self.species}/{self.species}.region.info.json"
                response = requests.get(url)
                self.region_data = response.json()["region_data"]
                self.region_type = response.json()["region_type"]

                # Cache the data to a JSON file
                with open(cache_file_path, "w") as cache_file:
                    json.dump(
                        {
                            "region_data": self.region_data,
                            "region_type": self.region_type,
                        },
                        cache_file,
                    )

    def _get_region_by_id(self, id):
        if self.region_data and id in self.region_data:
            return self.region_data[id]
        return None

    def _get_region_by_file(self, file):
        if self.region_data:
            for _, region in self.region_data.items():
                if region["file"] == file:
                    return region

    def _download_region(self, region):
        # TODO
        return f"test {region['file']}".encode()

    def _recover_tree(self, json_data, uid, tree, added_nodes):
        if uid in added_nodes:
            return tree.get_node(uid)

        item = json_data[str(uid)]
        parent_node = None
        parent_uid = item["parent_uid"]
        if parent_uid is not None:
            parent_node = self._recover_tree(
                json_data, str(parent_uid), tree, added_nodes
            )

        tree.create_node(
            identifier=str(item["uid"]),
            data={"acronym": item["acronym"], "name": item["name"]},
            parent=parent_node.identifier if parent_node else None,
        )
        added_nodes.add(uid)

        node = tree.get_node(uid)
        return node

    def region_tree(self):
        if not self.region_data:
            self._download_region_data()

        tree = Tree()
        added_nodes = set()
        for key in self.region_data.keys():
            self._recover_tree(self.region_data, str(key), tree, added_nodes)
        return tree

    def show_tree(self, tree):
        tree_structure = tree.show(stdout=False)
        display(Markdown(f"```\n{tree_structure}\n```"))



from digitalbrainsdk.api.api.core import Core

class DigitalBrain:
    def __init__(self, environment='PRODUCTION'):
        self.core = Core(environment)

    def get_environment(self):
        return self.core.get_environment()


import os
import pyswcloader
import pyswcloader.visualization


class SWCVisualization:
    def __init__(self):
        pass

    @staticmethod
    def view_swc(file_path):
        if not os.path.exists(file_path):
            raise FileNotFoundError(f"The file {file_path} does not exist.")
        
        return pyswcloader.visualization.plot_neuron_3d(file_path)




import unittest
from digitalbrainsdk.api.queries.region_api import RegionApi

class TestDigitalBrain(unittest.TestCase):
    def setUp(self):
        self.region_api=RegionApi(environment="DEBUG")
    
    def test_region_info_download(self):
        self.region_api._download_region_data()
        self.assertIsNotNone(self.region_api.region_data)
        self.assertIsNotNone(self.region_api.region_type)
    
    def test_get_region_by_id(self):
        region = self.region_api.get_region(id = "997")
        self.assertIsNotNone(region)
    
    def test_get_region_by_file(self):
        region = self.region_api.get_region(file="1.stl")
        self.assertIsNotNone(region)
    
    def test_region_tree(self):
        tree = self.region_api.region_tree()
        self.assertGreater(tree.depth(),0)


if __name__ == '__main__':
    unittest.main()


import requests
from digitalbrainsdk.api.api.core import Core
from digitalbrainsdk.config import ConfigManager
import os
import json

class SWCApi:
    def __init__(self, environment="PRODUCTION", species="mouse", project="pfc"):
        self.core = Core(environment)
        self.neuron_data = None
        self.neuron_type = None
        self.species = species
        self.project = project
        cache_dir_name = ConfigManager().get("CACHE", "CacheDir")
        self.cache_dir = None
        if not self.cache_dir:
            self.cache_dir = os.path.join(os.getcwd(), cache_dir_name, "swc")
        if not os.path.exists(self.cache_dir):
            os.makedirs(self.cache_dir)

    def get_swc(self, id=None, file=None):
        if not self.neuron_data:
            self._download_neuron_data()
        target_neuron = None
        if self.neuron_data:
            if id:
                target_neuron = self._get_swc_by_id(id)
            elif file:
                target_neuron = self._get_swc_by_file(file)

        # cache file path
        file_path = f"{self.cache_dir}/{target_neuron['file']}"

        # if exists return file path
        if os.path.exists(file_path):
            print(f"File already exists: {file_path}")
            return file_path

        # if not exists download and save
        if target_neuron:
            content = self.download_swc(target_neuron)
            with open(file_path, "wb") as f:
                f.write(content)
            return file_path
        return None

    def _download_neuron_data(self):
        # Check if species and project are valid
        if self.species and self.project:
            # Define the cache file path
            cache_file_path = (
                f"{self.cache_dir}/{self.species}_{self.project}_neuron.json"
            )

            # Check if the cache file exists
            if os.path.exists(cache_file_path):
                print(f"Loading cached data from {cache_file_path}")
                # Load the cached data
                with open(cache_file_path, "r") as cache_file:
                    cached_data = json.load(cache_file)
                    self.neuron_data = cached_data["neuron_data"]
                    self.neuron_type = cached_data["neuron_type"]
            else:
                # Fetch the data from the URL
                url = f"{self.core.base_url}/info/{self.species}/{self.project if self.project != 'pfc' else ''}/{self.species}.neuron.info.json"
                response = requests.get(url)
                self.neuron_data = response.json()["neuron_data"]
                self.neuron_type = response.json()["neuron_type"]

                # Cache the data to a JSON file
                with open(cache_file_path, "w") as cache_file:
                    json.dump(
                        {
                            "neuron_data": self.neuron_data,
                            "neuron_type": self.neuron_type,
                        },
                        cache_file,
                    )

    def _get_swc_by_id(self, id):
        # TODO
        # check if id is valid
        if self.neuron_data and id in self.neuron_data:
            return self.neuron_data[id]
        return None

    def _get_swc_by_file(self, file):
        # TODO
        # check if file is valid
        if self.neuron_data:
            for _, neuron in self.neuron_data.items():
                if neuron["file"] == file:
                    return neuron
        return None

    def download_swc(self, neuron):
        url = self._get_swc_download_url(neuron)
        if url:
            response = requests.get(url)
            if response.status_code == 200:
                return response.content
        return None

    def _get_swc_download_url(self, neuron):
        base_url = self.core.base_url + "/info/mouse"
        if self.project == "pfc":
            return f"{base_url}/pfc_neuron_download/{neuron['file'][:-4]}_reg.swc"
        elif self.project == "hy":
            return f"{base_url}/hy/hy_neuron_download/{neuron['file']}"
        elif self.project == "hipp":
            return f"{base_url}/hipp/hipp_neuron_download/{neuron['file']}"
        elif self.project == "pvh_oxt":
            return f"{base_url}/pvh_oxt/pvh_oxt_neuron_download/{neuron['file']}"
        elif self.project == "cea":
            return f"{base_url}/cea/cea_neuron_download/{neuron['file']}"
        else:
            return None

    def get_projects(self, species):
        if species == "mouse":
            return ["pfc", "hy", "hipp", "pvh_oxt", "cea"]
        elif species == "fish":
            return ["ei"]
        else:
            return None



import unittest
from digitalbrainsdk import DigitalBrain

class TestDigitalBrain(unittest.TestCase):

    def setUp(self):
        self.brain = DigitalBrain()
        print("setUp",self.brain.get_environment())

    def test_environment(self):
        self.brain = DigitalBrain(environment='DEBUG')
        self.assertEqual(self.brain.core.get_environment(), "DEBUG")

    # def test_process_data(self):
    #     data = "sample data"
    #     self.brain.initialize()
    #     result = self.brain.process_data(data)
    #     self.assertEqual(result, "processed sample data")

    # def test_get_results(self):
    #     self.brain.initialize()
    #     self.brain.process_data("sample data")
    #     results = self.brain.get_results()
    #     self.assertIsNotNone(results)

if __name__ == '__main__':
    unittest.main()


from digitalbrainsdk.config import ConfigManager


class Core:
    def __init__(self, environment="PRODUCTION"):
        self.environment = environment
        self.base_url = self._get_base_url()

    def _get_base_url(self):
        return ConfigManager().get(self.environment, "AppSRV")
     
    def get_environment(self):
        return self.environment




import unittest
from digitalbrainsdk.api.analyze.projection_api import AnalyzeApi

class TestDigitalBrain(unittest.TestCase):
    def setUp(self):
        self.projection_api=AnalyzeApi()
    
    def test_nrrd_download(self):
        nrrd_path = self.projection_api.download_nrrd()
        self.assertIsNotNone(nrrd_path)
    


if __name__ == '__main__':
    unittest.main()


import unittest
from digitalbrainsdk.api.queries.swc_api import SWCApi

class TestSWCApi(unittest.TestCase):
    
    def setUp(self):
        self.swc_api = SWCApi(species='mouse')

    def test_species(self):
        self.swc_api = SWCApi(species='mouse')
        self.assertEqual(self.swc_api.species, "mouse")
    
    def test_download_neuron_data(self):
        self.swc_api._download_neuron_data()
        self.assertIsNotNone(self.swc_api.neuron_data)
        self.assertIsNotNone(self.swc_api.neuron_type)
    
    def test_get_swc_by_id(self):
        self.swc_api._download_neuron_data()
        self.assertIsNotNone(self.swc_api._get_swc_by_id("1"))

    def test_get_swc_by_file(self):
        self.swc_api._download_neuron_data()
        self.assertIsNotNone(self.swc_api._get_swc_by_file("17099_002.swc"))

    def test_download_swc(self):
        self.swc_api._download_neuron_data()
        neuron = self.swc_api._get_swc_by_id("1")
        self.assertIsNotNone(self.swc_api.download_swc(neuron))
    
    def test_get_swc(self):
        result = self.swc_api.get_swc(id="1")
        self.assertIsNotNone(result)
    
    def test_project(self):
        self.swc_api = SWCApi(species='mouse', project='hipp')
        neuron = self.swc_api.get_swc(id="1")
        self.assertIsNotNone(neuron)
    
    def test_get_projects(self):
        projects = self.swc_api.get_projects(species='mouse')
        self.assertEqual(projects, ["pfc", "hy", "hipp", "pvh_oxt", "cea"])
        
if __name__ == '__main__':
    unittest.main()


class ApiModule:
    def __init__(self):
        self.name = "API Module"

    def get_data(self):
        return {"message": "Hello from the API!"}

    def post_data(self, data):
        return {"received": data}


def validate_data(data):
    # Implement validation logic for the input data
    if not isinstance(data, (list, dict)):
        raise ValueError("Data must be a list or a dictionary.")
    return True

def format_output(output):
    # Implement formatting logic for the output
    if isinstance(output, dict):
        return "\n".join(f"{key}: {value}" for key, value in output.items())
    return str(output)


import pyswcloader
import pyswcloader.brain
import os
from digitalbrainsdk.config import ConfigManager
import urllib.request


class AnalyzeApi:
    """
        A class to represent the Projection API.
        Attributes
        ----------
        data_path : str, optional
            The path to the swc data (default is None).
        annotation_path : str, optional
            The path for nrrd information (default is None).
        resolution : int, optional
            The resolution setting (default is 10).
        cores : int, optional
            The number of cores to use (default is 1).
    '"""

    def __init__(self, data_path=None, annotation_path=None, resolution=10, cores=1):
        """
        Constructs all the necessary attributes for the Projection API object.
        Parameters
        ----------
        data_path : str, optional
            The path to the swc data (default is None).
        annotation_path : str, optional
            The path for nrrd information (default is None).
        resolution : int, optional
            The resolution setting (default is 10).
        cores : int, optional
            The number of cores to use (default is 1).
        """
        self.data_path = data_path
        self.annotation_path = annotation_path
        self.resolution = resolution
        self.cores = cores
        self.cache_dir = None
        cache_dir_name = ConfigManager().get("CACHE", "CacheDir")
        if not self.cache_dir:
            self.cache_dir = os.path.join(os.getcwd(), cache_dir_name, "analyze")

    def batch_projection_length(self):
        """
        Batch projection length
        """
        if self.annotation_path is None:
            print("annotation_path is None, use the default annotation path")
            print(
                "Downloading nrrd from http://download.alleninstitute.org/informatics-archive/current-release/mouse_ccf/annotation/ccf_2017/annotation_10.nrrd"
            )
            self.download_nrrd()
        if self.data_path is None:
            raise ValueError("data_path is None, please provide a path to the swc data")

        annotation = pyswcloader.brain.read_nrrd(self.annotation_path)
        try:
            projection_length = pyswcloader.projection_batch.projection_length(
                data_path=self.data_path,
                annotation=annotation,
                resolution=self.resolution,
                cores=self.cores,
            )
            return projection_length
        except Exception as e:
            print(f"Error calculating projection length: {e}")
            return None

    def distance_morphology_matrix(self):
        """
        Distance morphology matrix
        """
        if self.data_path is None:
            raise ValueError("data_path is None, please provide a path to the swc data")

        try:
            scores = pyswcloader.distance.morphology_matrix(self.data_path, self.cores)
            return scores
        except Exception as e:
            print(f"Error calculating distance morphology matrix: {e}")
        return None

    def soma_distribution(self):
        """
        Soma distribution
        """
        if self.data_path is None:
            raise ValueError("data_path is None, please provide a path to the swc data")

        try:
            distribution = pyswcloader.swc.plot_soma_distribution(self.data_path)
            return distribution
        except Exception as e:
            print(f"Error calculating soma distribution: {e}")
        return None

    def download_nrrd(self):
        """
        Download nrrd
        """
        self.annotation_path = os.path.join(
            os.getcwd(), self.cache_dir, "nrrd", "annotation_10.nrrd"
        )
        if not os.path.exists(self.annotation_path):
            os.makedirs(os.path.dirname(self.annotation_path), exist_ok=True)

        try:
            urllib.request.urlretrieve(
                "http://download.alleninstitute.org/informatics-archive/current-release/mouse_ccf/annotation/ccf_2017/annotation_10.nrrd",
                self.annotation_path,
            )
        except Exception as e:
            print(f"Error downloading nrrd: {e}")
            return None
        print(f"Downloaded nrrd to {self.annotation_path}")
        return self.annotation_path

    def set_annotation_path(self, annotation_path):
        """
        Set the annotation path
        Parameters
        ----------
        annotation_path : str
            The path to the annotation file.
        """
        self.annotation_path = annotation_path

    def set_data_path(self, data_path):
        """
        Set the data path
        Parameters
        ----------
        data_path : str
            The path to the swc file.
        """
        self.data_path = data_path

    def set_resolution(self, resolution):
        """
        Set the resolution
        Parameters
        ----------
        resolution
            The resolution setting.
        """
        self.resolution = resolution

    def set_cores(self, cores):
        """
        Set the number of cores
        Parameters
        ----------
        cores
            The number of cores to use.
        """
        self.cores = cores



import configparser
import pkg_resources
"""
ConfigManager is a class to manage configuration files using configparser.
Attributes:
    config_file (str): The path to the configuration file.
    config (ConfigParser): An instance of ConfigParser to handle the configuration.
Methods:
    __init__(config_file='config.ini'):
        Initializes the ConfigManager with the specified configuration file.
    load_config():
        Loads the configuration from the config file.
    get(section, option, fallback=None):
        Retrieves the value for the given section and option from the configuration.
        If the section or option does not exist, returns the fallback value.
    set(section, option, value):
        Sets the value for the given section and option in the configuration.
        If the section does not exist, it is created.
        Saves the updated configuration to the file.
    save_config():
        Writes the current configuration to the config file.
"""

class ConfigManager:
    def __init__(self, config_file='config.ini'):
        self.config_file = config_file
        self.config = configparser.ConfigParser()
        self.load_config()

    def load_config(self):
        config_path = pkg_resources.resource_filename("digitalbrainsdk", "config.ini")
        self.config.read(config_path)

    def get(self, section, option, fallback=None):
        return self.config.get(section, option, fallback=fallback)

    def set(self, section, option, value):
        if not self.config.has_section(section):
            self.config.add_section(section)
        self.config.set(section, option, value)
        self.save_config()

    def save_config(self):
        with open(self.config_file, 'w') as configfile:
            self.config.write(configfile)


from setuptools import setup, find_packages

with open('requirements.txt', encoding='utf-8') as f:
    required = f.read().splitlines()

setup(
    name="digitalbrainsdk",
    version="0.0.7",
    author="XingLi",
    author_email="lihouxing@yeah.net",
    description="A Python SDK for Digital Brain analysis",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="",
    packages=find_packages(),
    include_package_data=True,
    package_data={
        "digitalbrainsdk": ["config.ini"],
    },
    install_requires=required,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)
